import logging
import sys


def configure_logging(log_level: str) -> None:
    """
    Configure the logging for the application.

    Args:
        log_level: The log level of the application. Value can be one of {debug, info, warn, error}.

    Raises:
        ValueError: If the log level is not one of the expected values.
    """
    root_logger = logging.getLogger()

    log_levels = {
        "debug": logging.DEBUG,
        "info": logging.INFO,
        "warn": logging.WARN,
        "error": logging.ERROR,
    }

    if log_level not in log_levels:
        raise ValueError(f"Invalid log level: {log_level}")

    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(log_levels[log_level])
    handler.setFormatter(formatter)

    root_logger.setLevel(log_levels[log_level])
    root_logger.addHandler(handler)



import logging
import os
import sys
import time

import typer
from prometheus_client import REGISTRY, start_http_server

from ._logging import configure_logging
from ._test_collector import TestCollector

logger = logging.getLogger("exporter")


def exporter(
    api_key: str = typer.Option(..., help="API Key for the account.", envvar="API_KEY"),
    tags: str = typer.Option(
        "",
        help="A comma separated list of tags used to filter tests returned from the api",
        envvar="TAGS",
    ),
    log_level: str = typer.Option(
        "info",
        help="The log level of the application. Value can be one of {debug, info, warn, error}",
        envvar="LOG_LEVEL",
    ),
    port: int = typer.Option(8000, envvar="PORT"),
    items_per_page=typer.Option(
        25,
        help="The number of items that the api will return on a page. This is a global option.",
        envvar="ITEMS_PER_PAGE",
    ),
):
    """
    The entry point for the exporter. This will start the exporter and begin collecting metrics.

    Args:
        api_key: The api key for the account.
        tags: A comma separated list of tags used to filter tests returned from the api.
        log_level: The log level of the application. Value can be one of {debug, info, warn, error}.
        port: The port that the exporter will listen on.
        items_per_page: The number of items that the api will return on a page. This is a global option.
    """

    try:
        configure_logging(log_level)

        logger.info(f"Starting web server on port: {port}")
        start_http_server(port)

        logger.info("Registering collectors.")
        test_collector = TestCollector(api_key, items_per_page, tags)
        REGISTRY.register(test_collector)

        while True:
            time.sleep(1)

    except BrokenPipeError:
        logger.fatal("Broken pipe error. Exiting.")
        devnull = os.open(os.devnull, os.O_WRONLY)
        os.dup2(devnull, sys.stdout.fileno())
        sys.exit(1)
    except Exception as e:
        logger.fatal(f"An unhandled exception occured: {e}")
        exit(1)
    finally:
        logger.debug("Shutting down.")
        sys.stdout.flush()


def run():
    typer.run(exporter)



import json
import logging
from time import sleep
from typing import Any

from statuscake import ApiClient
from statuscake.apis import MaintenanceWindowsApi, UptimeApi
from statuscake.exceptions import ApiValueError, ForbiddenException
from typing_extensions import NotRequired, TypedDict

logger = logging.getLogger("status_cake")


class DefaultPaginationParameters(TypedDict):
    """The default set of parameters used for pagination"""

    page: int
    limit: int


class PaginationParameters(TypedDict):
    """
    A generic type that is used as a hint for pagination args
    Consumers would inherit this type and add their own api parameters
    """

    pass


class ListUptimeTestParameters(PaginationParameters):
    """Parameters expected by the StatusCake API uptime endpoint"""

    tags: NotRequired[str]


class StatusCake:
    """
    A wrapper class for the StatusCake API client.
    """

    def __init__(self, api_key: str, per_page: int) -> None:
        """
        Args:
            api_key: [str] The StatusCake API key
            per_page: [int] The number of results to return per page
        """
        self.api_key: str = api_key
        self.per_page: int = int(per_page)

    def __get_api_client(self) -> ApiClient:
        """
        Returns an instance of the StatusCake API client

        Returns:
            ApiClient
        """
        return ApiClient(
            header_name="Authorization",
            header_value=f"Bearer {self.api_key}",
        )

    def __paginate_response(
        self, func: Any, args: PaginationParameters | None
    ) -> list[dict[str, Any]]:
        """
        A helper function that will paginate through the response of a given function.

        Args:
            func: [Any] The function to call
            args: [PaginationParameters | None] The arguments to pass to the function

        Returns:
            list[dict[str, Any]]
        """
        params: DefaultPaginationParameters = {"page": 1, "limit": self.per_page}
        args = args | params if args else params

        response = func(**params)
        metadata = response["metadata"]
        logger.debug(
            f"Received {metadata['total_count']} tests across {metadata['page_count']} page(s)"
        )

        data = response["data"]
        while params["page"] < metadata["page_count"]:
            params["page"] += 1
            logger.debug(f"Fetching page {params['page']} of {metadata['page_count']}")
            paged_response = func(**params)
            data.extend(paged_response["data"])

            sleep(1)

        return data

    def list_maintenance_windows(self) -> list[dict[str, Any]]:
        """
        Returns a list of maintenance windows

        Returns:
            list[dict[str, Any]]

        Raises:
            Exception: If an error occurs while fetching the maintenance windows
            ForbiddenException: If the API key does not have the required permissions
        """
        api_client = self.__get_api_client()

        try:
            maintenance_window_api: MaintenanceWindowsApi = MaintenanceWindowsApi(
                api_client
            )

            response = self.__paginate_response(
                maintenance_window_api.list_maintenance_windows,
                None,
            )
            return response

        except ForbiddenException as e:
            if e.body:
                message = json.loads(e.body)["message"]
                if (
                    message
                    and "Your current plan has no access to this feature" in message
                ):
                    logger.warn(
                        (
                            "Your current plan has no access to this feature or your account is "
                            "using legacy maintenance windows. Skipping maintenance window check."
                        )
                    )
                    return []

            # re-raise here because it might be genuine.
            raise e

        except Exception as e:
            logger.error(f"Error while fetching maintenance windows: {e}")
            raise e

    def list_tests(self, tags: str = "") -> list[dict]:
        """
        Returns a list of tests

        Args:
            tags: [str] A comma separated list of tags to filter by.

        Returns:
            list[dict[str, Any]]

        Raises:
            Exception: If an error occurs while fetching the tests
            ApiValueError: If the tags parameter is invalid
        """
        api_client = self.__get_api_client()

        try:
            uptime_api: UptimeApi = UptimeApi(api_client)
            params = ListUptimeTestParameters(tags=tags) if tags else None
            response = self.__paginate_response(
                uptime_api.list_uptime_tests,
                params,
            )
            return response

        # https://github.com/StatusCakeDev/statuscake-py/issues/8
        except ApiValueError as e:
            if "Invalid value for `total_count`" in str(e):
                logger.debug(f"No tests were found with the given tags '{tags}'.")
                return []

            raise e

        except Exception as e:
            logger.error(f"Error while fetching tests: {e}")
            raise e



#!/usr/bin/env python3

import logging

from prometheus_client.core import GaugeMetricFamily
from prometheus_client.registry import Collector

from ._status_cake import StatusCake

logger = logging.getLogger("test_collector")


def get_uptime_status(status: str) -> str:
    """
    Helper method that returns the appropriate status string for the
    uptime metric.

    Args:
        status: [str] The status of the test

    Returns:
        str
    """
    return "1" if status == "up" else "0"


def get_test_maintenance_status(id: str, tests_in_maintenance: list[str]) -> str:
    """
    Helper method that returns the appropriate status string for the maintenance metric.

    Args:
        id: [str] The id of the test
        tests_in_maintenance: [list[str]] A list of tests that are in maintenance

    Returns:
        str
    """
    return "1" if id in tests_in_maintenance else "0"


def get_tests_in_maintenance(maintenance_windows: list[dict[str, str]]) -> list[str]:
    """
    Get a unique list of tests that are in maintenance

    Args:
        maintenance_windows: [list[dict[str, str]]] The list of maintenance windows

    Returns:
        list[str]
    """
    t: list[str] = []

    # Return early if there are no maintenance windows otherwise we get an index out of range error
    if len(maintenance_windows) == 0:
        return t

    for i in maintenance_windows:
        t.extend(i["tests"])

    return list(set(t)) if len(t) > 0 else []


def transform(
    tests: list[dict[str, str]], tests_in_maintenance: list[str]
) -> list[dict[str, str]]:
    """
    Transform the tests into a format that prometheus can consume

    Args:
        tests: [list[dict[str, str]]] The list of tests to transform
        tests_in_maintenance: [list[str]] The list of tests that are in maintenance

    Returns:
        list[dict[str, str]]
    """
    logger.info("Transforming uptime test response to prometheus metrics")
    t: list[dict[str, str]] = []

    for i in tests:
        logger.debug(f"Transforming test id: {i['id']}")
        t.append(
            {
                "test_id": str(i["id"]),
                "test_type": str(
                    i["test_type"]
                ),  # This is random but we get an ApiAttributeError if we don't do this.
                "test_name": i["name"],
                "test_url": i["website_url"],
                "test_status_int": get_uptime_status(i["status"]),
                "test_uptime_percent": str(i["uptime"]),
                "maintenance_status_int": get_test_maintenance_status(
                    i["id"], tests_in_maintenance
                ),
            }
        )
        logger.debug(f"Transformed test id: {i['id']}")

    logger.debug(f"Test transformation complete. Returning {len(t)} metrics")
    return t if len(t) > 0 else []


class TestCollector(Collector):
    """The collector subclass responsible for gathering test metrics from the StatusCake API."""

    def __init__(self, api_key: str, per_page: int, tags: str):
        """
        Args:
            api_key: [str] The StatusCake API key
            per_page: [int] The number of tests to return per page
            tags: [str] The tags to filter the tests by
        """
        self.api_key: str = api_key
        self.per_page: int = per_page
        self.tags: str = tags

    def collect(self):
        """
        Collects metrics from the StatusCake API and returns them to the Prometheus client.

        Yields:
            [Generator] The generator that yields the metrics to the Prometheus client.
        """
        statuscake = StatusCake(self.api_key, self.per_page)

        logger.info("Collector started.")

        try:
            logger.debug("Fetching maintenance windows")
            maintenance = statuscake.list_maintenance_windows()

            tests_in_maintenance = get_tests_in_maintenance(maintenance)
            logger.info(
                f"Found {len(tests_in_maintenance)} tests that are in maintenance."
            )

            logger.debug("Fetching uptime tests")
            tests = statuscake.list_tests(self.tags)

            metrics = transform(tests, tests_in_maintenance)
            if len(metrics) == 0:
                logger.info("There are no test metrics to publish.")
                return

            # status_cake_test_info - gauge
            logger.info(f"Publishing {len(metrics)} test metric(s).")
            info_gauge = GaugeMetricFamily(
                "status_cake_test_info",
                "A basic listing of the tests under the current account.",
                labels=list(metrics[0].keys()),
            )

            for i in metrics:
                info_gauge.add_metric(list(i.values()), float(i["test_status_int"]))

            yield info_gauge

            # status_cake_test_uptime_percent - gauge
            logger.info(f"Publishing {len(metrics)} uptime metric(s).")
            uptime_gauge = GaugeMetricFamily(
                "status_cake_test_uptime_percent",
                "Tests and their uptime percentage",
                labels=["test_id"],
            )

            for i in metrics:
                uptime_gauge.add_metric([i["test_id"]], float(i["test_uptime_percent"]))

            yield uptime_gauge

        except Exception as e:
            import traceback

            # This should stop the expoter from crashing if there is an error.
            logger.fatal(f"A fatal error occurred: {e}")
            logger.debug(traceback.format_exc())

        finally:
            logger.info("Collector finished.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['status_cake_exporter']

package_data = \
{'': ['*']}

install_requires = \
['prometheus-client==0.15.0',
 'requests>=2.28.1,<3.0.0',
 'statuscake-py>=1.0.1b1,<2.0.0',
 'typer>=0.7.0,<0.8.0',
 'types-requests>=2.28.11.4,<3.0.0.0',
 'typing-extensions>=4.4.0,<5.0.0']

entry_points = \
{'console_scripts': ['status-cake-exporter = status_cake_exporter.app:run']}

setup_kwargs = {
    'name': 'status-cake-exporter',
    'version': '1.0.0',
    'description': '',
    'long_description': '# Status Cake Exporter\n\n![status-cake-exporter](https://github.com/chelnak/status-cake-exporter/actions/workflows/ci.yml/badge.svg)\n\n> :rotating_light: Container images have moved to ghcr.io/chelnak/status-cake-exporter\n\nStatus Cake Exporter is a Prometheus exporter for [StatusCake](https://www.statuscake.com/).\n\nMetrics are exposed on port 8000 when using the provided examples/manifest.yml](examples/manifest.yml) in Kubernetes, e.g.\n\n```sh\nhttp://status-cake-exporter.default.svc:8000\n```\n\n## Requirements\n\n* Python 3.10+\n* Docker\n* Kubernetes (optional)\n* Helm 3 (optional)\n\n## Usage\n\n| Setting                              | Required | Default |\n|--------------------------------------|----------|---------|\n| API_KEY                              | Yes      | Null    |\n| TAGS                                 | No       | Null    |\n| LOG_LEVEL                            | No       | info    | \n| PORT                                 | No       | 8000    |\n\n### Docker\n\nThe following will expose the exporter at `localhost:8000`:\n\n```sh\nexport API_KEY=xxxxxxxx\ndocker run -d -p 8000:8000 --env API_KEY ghcr.io/chelnak/status-cake-exporter:latest\n```\n\n### Kubernetes\n\nTo get up and running quickly, use [examples/manifest.yml](examples/manifest.yml) as an example. You will need to create a secret named `status-cake-api-token` containing your `API_KEY` first.\n\nOtherwise, you can use the Helm Chart provided in [chart/status-cake-exporter](chart/status-cake-exporter/README.md).\n\n### Grafana\n\nTo get up and running quickly, use [examples/grafana-example.json](examples/grafana-example.json) as an example. \n\n### Terminal\n\n```sh\nUsage: status-cake-exporter [OPTIONS]\n\nOptions:\n  --api-key TEXT         API Key for the account.  [env var: API_KEY;\n                         required]\n  --tags TEXT            A comma separated list of tags used to filter tests\n                         returned from the api  [env var: TAGS]\n  --log-level TEXT       The log level of the application. Value can be one of\n                         {debug, info, warn, error}  [env var: LOG_LEVEL;\n                         default: info]\n  --port INTEGER         [env var: PORT; default: 8000]\n  --items-per-page TEXT  The number of items that the api will return on a\n                         page. This is a global option.  [env var:\n                         ITEMS_PER_PAGE; default: 25]\n  --help                 Show this message and exit\n```\n\n## Metrics\n\n| Name| Type | Description |\n|-----|------|-------------|\n| status_cake_test_info | Gauge |A basic listing of the tests under the current account. |\n| status_cake_test_uptime_percent | Gauge | Tests and their uptime percentage |\n\n## Prometheus\n\nPrometheus config needs to be updated to see the new exported. Use the following scrape config as an example:\n\n```Yaml\nscrape_configs:\n    - job_name: status-cake-exporter\n    honor_timestamps: true\n    scrape_interval: 10m\n    scrape_timeout: 1m\n    metrics_path: /\n    scheme: http\n    static_configs:\n    - targets:\n        - status-cake-exporter.default.svc:8000\n```\n\n## Grafana\n\nData collected by Prometheus can be easily surfaced in Grafana.\n\nUsing the [Statusmap panel](https://grafana.com/grafana/plugins/flant-statusmap-panel) by [flant](https://github.com/flant/grafana-statusmap) you can create a basic status visualization based on uptime percentage:\n\n![grafana](examples/grafana.png)\n\n### PromQL\n\n```PromQL\nstatus_cake_test_info * on(test_id) group_right(test_name) status_cake_test_uptime_percent\n```\n\n## Development\n\nThis repository uses [Tilt](https://tilt.dev) for rapid development on Kubernetes.\n\nTo use this, run:\n\n```sh\ncd chart/status-cake-exporter\ntilt up\n```\n\nTilt will reload your environment when it detects changes to your code.\n\nNote: You will need to provide valid credentials for StatusCake in your `Tiltfile` for this to work.\n',
    'author': 'Craig Gumbley',
    'author_email': 'craiggumbley@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
